@@ -31,7 +31,9 @@ class MainWindow(hildon.StackableWindow):
     def __init__(self):
         hildon.StackableWindow.__init__(self)
         self.program = hildon.Program.get_instance()
-        self.connect("delete_event", lambda a, b: gtk.main_quit())
+        self.program.set_can_hibernate(True)
+
+        self.connect("delete_event", lambda a, b: self.quit())
         self.program.add_window(self)
 
         self.layout = gtk.Fixed()
@@ -41,7 +43,9 @@ class MainWindow(hildon.StackableWindow):
         self.layout.put(self.graph, 0, 0)
 
         gtk.set_application_name("battery-eye")
+        self.setupMenu()
 
+    def setupMenu(self):
         self.menu = hildon.AppMenu()
         self.filters = []
         self.filters.append(hildon.GtkRadioButton(gtk.HILDON_SIZE_AUTO, None))
@@ -59,29 +63,76 @@ class MainWindow(hildon.StackableWindow):
         self.filters[2].set_label('mV')
         self.filters[2].set_mode(False)
         self.filters[2].connect("clicked", lambda button: self.graph.changeDataset('hal.battery.voltage.current'))
-
-#        self.filters.append(hildon.GtkRadioButton(gtk.HILDON_SIZE_AUTO, None))
-#        self.filters[3].set_label('1 day')
-#        self.filters[3].set_mode(False)
-#        self.filters[3].connect("clicked",
-#                                lambda button: self.graph.changeVisibleHours(25))
-
-#        self.filters.append(hildon.GtkRadioButton(gtk.HILDON_SIZE_AUTO, self.filters[3]))
-#        self.filters[4].set_label('3 days')
-#        self.filters[4].set_mode(False)
-#        self.filters[4].connect("clicked", lambda button: self.graph.changeVisibleHours(25*3))
-
-#        self.filters.append(hildon.GtkRadioButton(gtk.HILDON_SIZE_AUTO, self.filters[4]))
-#        self.filters[5].set_label('7 days')
-#        self.filters[5].set_mode(False)
-#        self.filters[5].connect("clicked", lambda button: self.graph.changeVisibleHours(25*7))
-
+        
         for f in self.filters:
             self.menu.add_filter(f)
 
+        self.zoomButton = hildon.PickerButton(gtk.HILDON_SIZE_AUTO, hildon.BUTTON_ARRANGEMENT_HORIZONTAL)
+        self.zoomButton.set_title('Zoom')
+        self.zoomSelector = hildon.TouchSelector(text = True)
+        self.zoomSelector.append_text('1 day')
+        self.zoomSelector.append_text('2 days')
+        self.zoomSelector.append_text('3 days')
+        self.zoomSelector.append_text('5 days')
+        self.zoomSelector.append_text('7 days')
+        self.zoomSelector.set_active(0, 0)
+        self.zoomSelector.connect("changed", self.changeZoom)
+        self.zoomButton.set_selector(self.zoomSelector)
+
+        self.menu.append(self.zoomButton)
+
+        self.deleteButton = hildon.Button(gtk.HILDON_SIZE_AUTO, hildon.BUTTON_ARRANGEMENT_HORIZONTAL)
+        self.deleteButton.set_label('Delete data...')
+        self.deleteButton.connect("clicked", self.deleteData)
+
+        self.menu.append(self.deleteButton)
+
         self.menu.show_all()
         self.set_app_menu(self.menu)
 
+    def deleteData(self, widget):
+        dlg = gtk.Dialog()
+        dlg.set_title("Delete all graph data older than")
+
+        selector = hildon.TouchSelector(text = True)
+        selector.append_text('3 days')
+        selector.append_text('7 days')
+        selector.append_text('14 days')
+        selector.append_text('28 days')
+        selector.append_text('Delete all')
+        selector.connect("changed", self.doDelete, dlg)
+        dlg.vbox.pack_start(selector, True, True)
+        dlg.vbox.set_size_request(300, 300)
+        dlg.set_transient_for(self)
+
+        dlg.show_all()
+        dlg.run()
+        dlg.destroy()
+
+    def doDelete(self, selector, user_data, parent):
+        if selector.get_current_text() == 'Delete all':
+            msg = 'All graph data will be deleted. Continue?'
+        else:
+            msg = 'Graph data older than %s will be deleted. Continue?' % selector.get_current_text()
+        note = hildon.hildon_note_new_confirmation(parent, msg)
+        response = note.run()
+        if response == gtk.RESPONSE_OK:
+            # Delete
+            parent.response(gtk.RESPONSE_OK)
+            now = int(time.time())
+            values = {'3 days': now - 3 * 86400,
+                      '7 days': now - 7 * 86400,
+                      '14 days': now - 14 * 86400,
+                      '28 days': now - 28 * 86400,
+                      'Delete all': now + 1}
+            dataStorage.deleteObservations(values[selector.get_current_text()])
+            self.graph.updateData()
+        note.destroy()
+       
+    def changeZoom(self, selector, user_data):
+        values = {'1 day': 25, '2 days': 25*2, '3 days': 25*3, '5 days': 25*5, '7 days': 25*7}
+        self.graph.changeVisibleHours(values[selector.get_current_text()])
+
     def run(self):
         self.show_all()
         self.graph.jump_to(self.graph.drawAreaInfo.canvasSize[0]-1, 0)
@@ -91,6 +142,11 @@ class MainWindow(hildon.StackableWindow):
             print ""
             print "Ctrl-C"
 
+    def quit(self):
+        # set_app_menu(None) gets rid of glib assertion errors on exit
+        self.set_app_menu(None)
+        gtk.main_quit()
+
 class DrawAreaInfo(object):
     def __init__(self, canvasSize, drawAreaSize, offset, windowSize):
         self.canvasSize = canvasSize
@@ -157,6 +213,8 @@ class Graph(hildon.PannableArea):
         self.dataset = 'hal.battery.charge_level.percentage'
         self.visibleHours = 25
 
+        self.requestJump = None
+
         self.updateData()
 
     def changeDataset(self, dataset):
@@ -166,7 +224,7 @@ class Graph(hildon.PannableArea):
     def changeVisibleHours(self, visibleHours):
         self.visibleHours = visibleHours
         self.updateData()
-        self.jump_to(self.drawAreaInfo.canvasSize[0]-1, 0)
+        self.requestJump = self.drawAreaInfo.canvasSize[0]-1
 
     def updateData(self):
         if self.dataset == 'hal.battery.charge_level.percentage':
@@ -185,14 +243,33 @@ class Graph(hildon.PannableArea):
             yLabelsInterval = 100
             yLabels = ['3600mV', '3700mV', '3800mV', '3900mV', '4000mV', '4100mV', '4200mV']
 
-        rawData = dataStorage.getObservations(self.dataset)
+        now = int(time.time())
 
-        self.xMax = int(time.time()) + 2*3600
+        rawData = dataStorage.getObservationsWithBreaks(self.dataset, now - 30*86400, now)
 
-        if len(rawData) < 1:
+        self.xMax = now + 2*3600
+
+        firstObservationTime = None
+        for segment in rawData:
+            if len(segment) > 0:
+                firstObservationTime = segment[0][0]
+                break
+
+        if firstObservationTime == None:
             self.xMin = self.xMax - self.visibleHours*3600
         else:
-            self.xMin = min(rawData[0][0], self.xMax - self.visibleHours*3600)
+            self.xMin = min(firstObservationTime, self.xMax - self.visibleHours*3600)
+
+        if self.visibleHours < 48:
+            self.xGridInterval = 1
+            self.xLabelInterval = 2
+        elif self.visibleHours < 100:
+            self.xGridInterval = 2
+            self.xLabelInterval = 6
+        else:
+            self.xGridInterval = 6
+            self.xLabelInterval = 24
+            
 
         areaWidth = max((self.xMax - self.xMin) / 3600 * (self.graphWidth/self.visibleHours), self.graphWidth - 10)
 
@@ -204,12 +281,13 @@ class Graph(hildon.PannableArea):
         self.pixmap = gtk.gdk.Pixmap(None, areaWidth, self.graphHeight, 16)
         self.area.set_size_request(areaWidth, self.graphHeight)
 
-        self.data = self.rescaleData(rawData, self.drawAreaInfo, self.xMin, self.xMax, yMin, yMax)
+        self.datas = []
+        for segment in rawData:
+            self.datas.append(self.rescaleData(segment, self.drawAreaInfo, self.xMin, self.xMax, yMin, yMax))
         self.yGrid = self.yGridSegments(self.drawAreaInfo, yMin, yMax, yLabelsInterval)
-        self.xGrid = self.xGridSegments(self.drawAreaInfo, self.xMin, self.xMax)
+        self.xGrid = self.xGridSegments(self.drawAreaInfo, self.xMin, self.xMax, self.xGridInterval)
         self.yLabels = self.createYLabels(yLabels)
-        self.xLabels = self.createXLabels(self.xMin, self.xMax)
-        self.charge = self.chargeInfo()
+        self.xLabels = self.createXLabels(self.xMin, self.xMax, self.xLabelInterval)
 
         gc = self.pixmap.new_gc()
 
@@ -224,11 +302,14 @@ class Graph(hildon.PannableArea):
         self.pixmap.draw_segments(gc, self.xGrid)
 
         gc.foreground = self.dataColor
-        self.pixmap.draw_lines(gc, self.data)
+        for d in self.datas:
+            if len(d):
+                # gtk outputs assertion error if lines is an empty array
+                self.pixmap.draw_lines(gc, d)
 
         gc.foreground = self.labelsColor
         self.drawYLabels(gc, self.yLabels, self.yGrid)
-        self.drawXLabels(gc, self.xLabels)
+        self.drawXLabels(gc, self.xLabels, self.xLabelInterval)
 
         self.area.queue_draw()
 
@@ -243,6 +324,9 @@ class Graph(hildon.PannableArea):
                                     area.x, area.y,
                                     area.x, area.y,
                                     area.width, area.height)
+        if (self.requestJump != None):
+            self.jump_to(self.requestJump, 0)
+            self.requestJump = None
 
     def moveLabels(self, adjustment):
         area = self.drawYLabels(self.pixmap.new_gc(), self.yLabels, self.yGrid)
@@ -305,18 +389,21 @@ class Graph(hildon.PannableArea):
             ret.append((areaInfo.offset[0], y, areaInfo.drawAreaSize[0]-1, y))
         return ret
 
-    def chargeInfo(self):
-        rawData = dataStorage.getObservations(('hal.battery.rechargeable.is_charging',
-                                               'hal.battery.rechargeable.is_discharging'))
-#        print rawData
+#    def chargeInfo(self, now):
+#        rawData = dataStorage.getObservationsWithBreaks(('hal.battery.rechargeable.is_charging',
+#                                                         'hal.battery.rechargeable.is_discharging'),
+#                                                         now - 30*86400, now)
+#        charges = []
+#        fulls = []
+#        for segment in 
         
-    def createXLabels(self, xMin, xMax):
+    def createXLabels(self, xMin, xMax, modulo):
         hours = self.getHoursInInterval(xMin, xMax, 1)
         pContext = self.area.create_pango_context()
         pContext.set_font_description(pango.FontDescription('sans normal 10'))
         labelPixmaps = []
         for hour in hours:
-            if hour.hour % 4 == 0:
+            if hour.hour % modulo == 0:
                 layout = pango.Layout(pContext)
                 if hour.hour == 0:
                     layout.set_text(hour.strftime('%d. %b'))
@@ -331,11 +418,11 @@ class Graph(hildon.PannableArea):
                 labelPixmaps.append(pixmap)
         return labelPixmaps
 
-    def drawXLabels(self, gc, pixmaps):
+    def drawXLabels(self, gc, pixmaps, modulo):
         hours = self.getHoursInInterval(self.xMin, self.xMax, 1)
         labeledHours = []
         for hour in hours:
-            if hour.hour % 4 == 0:
+            if hour.hour % modulo == 0:
                 labeledHours.append(hour)
         y = self.drawAreaInfo.offset[1] + self.drawAreaInfo.drawAreaSize[1] + 1
         for pixmap, hour in zip(pixmaps, labeledHours):
